function [out, season, beta] = deseason_harmonic(in, K, L)

%
%[deseasoned_data, season, beta] = deseason_harmonic(data, K, L)
% 
% Subtracts the seasonal cycle (season) from the data (data). Season
% is calculated by fitting K harmonics of the annual cycle (as well as the
% mean) to the data. Assumes on year has L elements (i.e., 365 for daily data,
% 73 for pentad data, 52 for weekly data, etc.).
% Outputs the deseasonalized data, the season, and the fitting parameters (beta)
%
% Written by Eric Oliver, Dalhousie University, 2007-2011
%

n = length(in);
time = [1:n]'./L;

% set up mean and harmonics to fit data
P = ones(n,1);
for k=1:K
    P=[P cos(k*2*pi*time) sin(k*2*pi*time)];
end 

% Remove seasonal cycle by harmonic regression
beta = inv(P'*P)*P'*in;

season = P*beta;
out = in - season;
