function [out, season, beta] = deseason_harmonic_nans(in, K, L)

%
%[deseasoned_data, season, beta] = deseason_harmonic_nans(data, K, L)
% 
% Subtracts the seasonal cycle (season) from the data (data) which contains
% some NaN values. Season
% is calculated by fitting K harmonics of the annual cycle (as well as the
% mean) to the data. Assumes on year has L elements (i.e., 365 for daily data,
% 73 for pentad data, 52 for weekly data, etc.) and that mod(data,L)=0.
% Outputs the deseasonalized data, the season, and the fitting parameters
% (beta)
%
% Note: Works only for daily data

n = length(in);
time = [1:n]'./L;
notnans = ~isnan(in);

% set up mean and harmonics to fit data
P = ones(n,1);
for k=1:K
    P=[P cos(k*2*pi*time) sin(k*2*pi*time)];
end 

% Remove seasonal cycle by harmonic regression and using only non-NaN data points
beta = inv(P(notnans,:)'*P(notnans,:))*P(notnans,:)'*in(notnans);

season = P*beta;
out = in - season;
